!----------------------------------------------------------------------
!----------------------------------------------------------------------
!   lsa :   Laser with Saturable Absorber
!----------------------------------------------------------------------
!----------------------------------------------------------------------
 
      SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP) 
!     ---------- ----

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM, ICP(*), IJAC
      DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
      DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)
      DOUBLE PRECISION, INTENT(INOUT) :: DFDU(NDIM,NDIM), DFDP(NDIM,*)
      DOUBLE PRECISION alpha, Q, gammaR, gamma1, gamma2, z, beta
      DOUBLE PRECISION c, v, w
      DOUBLE PRECISION I, b1, z1, Omega, g1, g2, dOdc
 
      alpha = PAR(1)
      Q     = PAR(2)
      gammaR= PAR(4)
      gamma1= PAR(5)
      gamma2= PAR(6)
      z     = PAR(7)
      beta  = PAR(8)

      c = U(1)
      v = U(2)
      w = U(3)

      I = DEXP(c)
      b1 = 1 + 2*beta*I
      z1 = z + 1
      Omega= z1*I*gammaR / ( gammaR*z1**2 + 2*z*I )
      g1 = Omega + gamma1
      g2 = Omega + gamma2

      F(1) = -1 + z1*Omega*(w-v)/(z*I) - alpha/b1
      F(2) = -g1*v + Omega*w
      F(3) =  Omega*v - g2*w + z*gamma2*Q

      IF(IJAC.EQ.0)RETURN

      dOdc = z1**3*gammaR**2*I/(gammaR*z1**2 + 2*z*I)**2

      DFDU(1,1) = z1*(w-v)*(dOdc-Omega)/(z*I) + 2*alpha*beta*I/b1**2
      DFDU(1,2) = -z1*Omega/(z*I)
      DFDU(1,3) =  z1*Omega/(z*I)

      DFDU(2,1) =  (w-v)*dOdc
      DFDU(2,2) = -g1
      DFDU(2,3) =  Omega

      DFDU(3,1) =  (v-w)*dOdc
      DFDU(3,2) =  Omega
      DFDU(3,3) = -g2

      IF(IJAC.EQ.1)RETURN

!     Derivatives w.r.t. alpha
      DFDP(1,1) = -1/b1
      DFDP(2,1) =  0
      DFDP(3,1) =  0

!     Derivatives w.r.t. Q
      DFDP(1,2) = 0
      DFDP(2,2) = 0
      DFDP(3,2) = z*gamma2

!     Derivatives w.r.t. beta
      DFDP(1,8) = 2*alpha*I/b1**2
      DFDP(2,8) = 0
      DFDP(3,8) = 0

      RETURN
      END SUBROUTINE FUNC
!----------------------------------------------------------------------

      SUBROUTINE STPNT(NDIM,U,PAR) 
!     ---------- -----

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM
      DOUBLE PRECISION, INTENT(INOUT) :: U(NDIM),PAR(*)
      DOUBLE PRECISION alpha, Q, gammaR, gamma1, gamma2, z, beta

      alpha  = 0.75
      Q      = 3.0
      gammaR = 0.2205
      gamma1 = 0.0252
      gamma2 = 0.00315
      z      = 10.
      beta   = 200.

      PAR(1) = alpha
      PAR(2) = Q
      PAR(4) = gammaR
      PAR(5) = gamma1
      PAR(6) = gamma2
      PAR(7) = z
      PAR(8) = beta

      U(1) =-2.85646E+00   
      U(2) = 2.13812E+00   
      U(3) = 1.28950E+01

      END SUBROUTINE STPNT
!----------------------------------------------------------------------
!----------------------------------------------------------------------
      SUBROUTINE PVLS(NDIM,U,PAR)
!     ---------- ----
!
      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM
      DOUBLE PRECISION, INTENT(IN) :: U(NDIM)
      DOUBLE PRECISION, INTENT(INOUT) :: PAR(*)
      DOUBLE PRECISION, EXTERNAL :: GETP
      DOUBLE PRECISION a1, a2, a3

       a1 = GETP('NRM',1,U)
       a2 = GETP('NRM',2,U)
       a3 = GETP('NRM',3,U)
       PAR(9) = DSQRT(a1**2 + a2**2 + a3**2)

      END SUBROUTINE PVLS
!----------------------------------------------------------------------
!----------------------------------------------------------------------
      SUBROUTINE BCND 
      RETURN 
      END 
! 
      SUBROUTINE ICND 
      RETURN 
      END 
! 
      SUBROUTINE FOPT 
      RETURN 
      END 
!----------------------------------------------------------------------
!----------------------------------------------------------------------
