!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
!   lsa :   Coupled Lasers with Saturable Absorber
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
 
 SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP) 
!---------- ---- 

 IMPLICIT NONE
 INTEGER, INTENT(IN) :: NDIM, ICP(*), IJAC
 DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
 DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)
 DOUBLE PRECISION, INTENT(INOUT) :: DFDU(NDIM,NDIM), DFDP(NDIM,*) 

 DOUBLE PRECISION I1, I2, G1, G2, Q1, Q2, R1, R2
 DOUBLE PRECISION a, A1, A2, B, eps, c, d
 
   a  = PAR(1)
   A1 = PAR(2)
   A2 = A1	!Note that A1=A2 
   B  = PAR(4)
   eps= PAR(5)
   c  = PAR(6)
   d  = PAR(7)

! Variables:
! ----------
   I1 = EXP(U(1))
   G1 = U(2)
   I2 = EXP(U(3))
   G2 = U(4)

   Q1 = 1 + a*(I1+c*I2)
   Q2 = 1 + a*(I2+c*I1)

   R1 =   B/Q1
   R2 = d*B/Q2

   F(1) = ( G1 - R1 - 1)
   F(2) = eps * ( A1 - G1*(1+I1) )
   F(3) = ( G2 - R2 - 1)
   F(4) = eps * ( A2 - G2*(1+I2) )

 IF(IJAC.EQ.0)RETURN

   DFDU(1,1) = B*a*I1/Q1**2
   DFDU(1,2) = 1
   DFDU(1,3) = B*a*c*I2/Q1**2
   DFDU(1,4) = 0

   DFDU(2,1) = -eps*G1*I1
   DFDU(2,2) = -eps*(1+I1)
   DFDU(2,3) = 0
   DFDU(2,4) = 0

   DFDU(3,1) = d*B*a*c*I1/Q2**2
   DFDU(3,2) = 0
   DFDU(3,3) = d*B*a*I2/Q2**2
   DFDU(3,4) = 1

   DFDU(4,1) = 0
   DFDU(4,2) = 0
   DFDU(4,3) = -eps*G2*I2
   DFDU(4,4) = -eps*(1+I2)

 IF(IJAC.EQ.1)RETURN

   DFDP(1,2) = 0
   DFDP(2,2) = eps
   DFDP(3,2) = 0
   DFDP(4,2) = eps

   DFDP(1,6) = B*a*I2/Q1**2
   DFDP(2,6) = 0
   DFDP(3,6) = d*B*a*I1/Q2**2
   DFDP(4,6) = 0

 RETURN
 END 
!----------------------------------------------------------------------
!----------------------------------------------------------------------
 SUBROUTINE STPNT(NDIM,U,PAR,T)
!---------- -----

 IMPLICIT NONE
 INTEGER, INTENT(IN) :: NDIM
 DOUBLE PRECISION, INTENT(INOUT) :: U(NDIM),PAR(*)
 DOUBLE PRECISION, INTENT(IN) :: T

 DOUBLE PRECISION I1, I2, G1, G2, a, A1, A2, B, eps, c, d

   B  = 2.d0
   a  = 2.d0
   A1 = 6.d0
   eps= 0.06d0
   c  = 0.001d0
   d  = 1.d0

   PAR(1) = a
   PAR(2) = A1
   PAR(4) = B
   PAR(5) = eps
   PAR(6) = c
   PAR(7) = d

   U(1) = 1.35764d0
   U(2) = 1.22774d0
   U(3) = 1.35764d0
   U(4) = 1.22774d0

 RETURN
 END
!----------------------------------------------------------------------
!----------------------------------------------------------------------
 SUBROUTINE PVLS(NDIM,U,PAR)
!---------- -----

 IMPLICIT NONE
 INTEGER, INTENT(IN) :: NDIM
 DOUBLE PRECISION, INTENT(IN) :: U(NDIM)
 DOUBLE PRECISION, INTENT(INOUT) :: PAR(*)
 DOUBLE PRECISION, EXTERNAL :: GETP,GETU2

 DOUBLE PRECISION Int1, Int2, Int3, Int4

!  PAR(8) keeps track of U(1) ( = log(I1) ) for stopping purposes:
   PAR(8) = GETP('BV0',1,U)

!  PAR(9) contains the integral of (G1 + G2):
   Int2 = GETP('INT',2,U)
   Int4 = GETP('INT',4,U)
   PAR(9) = Int2 + Int4

 RETURN
 END
!----------------------------------------------------------------------
!----------------------------------------------------------------------
 SUBROUTINE BCND 
 RETURN 
 END 
 
 SUBROUTINE ICND 
 RETURN 
 END 
 
 SUBROUTINE FOPT 
 RETURN 
 END 
!----------------------------------------------------------------------
!----------------------------------------------------------------------

