!----------------------------------------------------------------------
!----------------------------------------------------------------------
!   lsa :   Laser with Saturable Absorber
!----------------------------------------------------------------------
!----------------------------------------------------------------------
 
      SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP) 
!     ---------- ----

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM, ICP(*), IJAC
      DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
      DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)
      DOUBLE PRECISION, INTENT(INOUT) :: DFDU(NDIM,NDIM), DFDP(NDIM,*)
      DOUBLE PRECISION alpha, Q, gammaR, gamma1, gamma2, z, beta
      DOUBLE PRECISION c, v, w
      DOUBLE PRECISION I, b1, z1, Omega, g1, g2, dOdc
      DOUBLE PRECISION freq, amp, Period, x, y, ss
      INTEGER k
 
      alpha = PAR(1)
      Q     = PAR(2)
      gammaR= PAR(4)
      gamma1= PAR(5)
      gamma2= PAR(6)
      z     = PAR(7)
      beta  = PAR(8)
      freq  = PAR(9)
      amp   = PAR(10)
      Period= PAR(11)

      c = U(1)
      v = U(2)
      w = U(3)
      x = U(4)
      y = U(5)

      I = DEXP(c)
      b1 = 1 + 2*beta*I
      z1 = z + 1
      Omega= z1*I*gammaR / ( gammaR*z1**2 + 2*z*I )
      g1 = Omega + gamma1
      g2 = Omega + gamma2

      F(1) = -1 + amp*x + z1*Omega*(w-v)/(z*I) - alpha/b1
      F(2) = -g1*v + Omega*w
      F(3) =  Omega*v - g2*w + z*gamma2*Q

! The added oscillator:
      ss = x**2 + y**2
      F(4) =  x + freq*y - x*ss
      F(5) = -freq*x + y - y*ss

      DO k=1,NDIM
        F(k) = Period*F(k)
      ENDDO

      END SUBROUTINE FUNC
!----------------------------------------------------------------------

      SUBROUTINE BCND(NDIM,PAR,ICP,NBC,U0,U1,FB,IJAC,DBC)
!     ---------- ----

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM, ICP(*), NBC, IJAC
      DOUBLE PRECISION, INTENT(IN) :: PAR(*), U0(NDIM), U1(NDIM)
      DOUBLE PRECISION, INTENT(OUT) :: FB(NBC)
      DOUBLE PRECISION, INTENT(INOUT) :: DBC(NBC,*)
      INTEGER k

       DO k=1,NDIM
         FB(k)=U1(k) - U0(k)
       ENDDO

       FB(NDIM+1)=U0(4)

      END SUBROUTINE BCND
!----------------------------------------------------------------------

      SUBROUTINE STPNT(NDIM,U,PAR,t) 
!     ---------- -----

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM
      DOUBLE PRECISION, INTENT(INOUT) :: U(NDIM),PAR(*)
      DOUBLE PRECISION, INTENT(IN) :: t
      DOUBLE PRECISION alpha, Q, gammaR, gamma1, gamma2, z, beta
      DOUBLE PRECISION freq, amp, tpi, Period

      tpi    = 8*DATAN(1.d0)

      alpha  = 0.	!Note that alpha is zero!
      Q      = 1.12d0
      gammaR = 0.02205
      gamma1 = 0.00252
      gamma2 = 0.000315
      z      = 10.
      beta   = 200.
      freq   = 0.1*tpi/(2*15.8730d0)
      amp    = 0.0
      Period = tpi/freq

      PAR(1) = alpha
      PAR(2) = Q
      PAR(4) = gammaR
      PAR(5) = gamma1
      PAR(6) = gamma2
      PAR(7) = z
      PAR(8) = beta
      PAR(9) = freq
      PAR(10)= amp 
      PAR(11)= Period

      U(1) =-7.9259137454942d0
      U(2) = 0.1303244005700d0
      U(3) =10.1574047954938d0

      U(4)=SIN(tpi*t)
      U(5)=COS(tpi*t)

      END SUBROUTINE STPNT
!----------------------------------------------------------------------
!----------------------------------------------------------------------
      SUBROUTINE PVLS(NDIM,U,PAR)
!     ---------- ----
!
      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM
      DOUBLE PRECISION, INTENT(IN) :: U(NDIM)
      DOUBLE PRECISION, INTENT(INOUT) :: PAR(*)
      DOUBLE PRECISION, EXTERNAL :: GETP
      DOUBLE PRECISION a1, a2, a3

       a1 = GETP('NRM',1,U)
       a2 = GETP('NRM',2,U)
       a3 = GETP('NRM',3,U)
       PAR(12) = DSQRT(a1**2 + a2**2 + a3**2)

      END SUBROUTINE PVLS
!----------------------------------------------------------------------
!----------------------------------------------------------------------
      SUBROUTINE ICND 
      RETURN 
      END 
! 
      SUBROUTINE FOPT 
      RETURN 
      END 
!----------------------------------------------------------------------
!----------------------------------------------------------------------
